import code


variables = globals().copy()
variables.update(locals())
shell = code.InteractiveConsole(variables)


# Video
path = "C:/Users/migue/Documents/U/4to/TFG/Videos/Pez1.mp4"
shell.push("from Video import Video")
shell.push(f"v = Video(\"{path}\")")



shell.interact()


import cv2 as cv

default_params = {
  "filterSize": (9, 9)
}

def top_hat(frame, params = default_params):
    """Performs a top-hat operation on the given frame and returns the result."""
    # Getting the kernel to be used in Top-Hat
    filterSize = params["filterSize"]

    kernel = cv.getStructuringElement(cv.MORPH_RECT,
                                      filterSize)

    # Applying the Top-Hat operation
    filtered_frame = cv.morphologyEx(frame,
                                 cv.MORPH_TOPHAT,
                                 kernel)

    return filtered_frame



from .filters.top_hat import top_hat
import cv2 as cv

default_options = {
    "filter": top_hat,
    "filter_params": {
        "filterSize": (9, 9),
    }
}


def homogeneize_background(frame, options = default_options):
    filter = options["filter"]
    filter_params = options["filter_params"]
    return filter(frame, filter_params)


def frame_to_gray(frame):
    gray = cv.cvtColor(frame, cv.COLOR_BGR2GRAY)
    return gray


def denoise(frame):
    return frame


def preprocess_frame(frame):
    homogeneized = homogeneize_background(frame)
    gray = frame_to_gray(homogeneized)
    denoised = denoise(gray)
    return denoised



import cv2 as cv
from .preprocess_frame import preprocess_frame


def detect_objects_in_frame(frame):
    preprocessed_frame = preprocess_frame(frame)

    thresh = cv.threshold(preprocessed_frame, 20, 255, cv.THRESH_BINARY)[1]

    cnts, hierarchy = cv.findContours(thresh, cv.RETR_EXTERNAL, cv.CHAIN_APPROX_SIMPLE)
    return cnts
    
        
def draw_contours(frame, contours):
    for cnt in contours:
      (x, y), radius = cv.minEnclosingCircle(cnt)
      center = (int(x), int(y))
      radius = int(radius)
      radius = int(radius + (radius * 2))
      
      area = "{:6.2f}".format(cv.contourArea(cnt))
      font = cv.FONT_HERSHEY_SIMPLEX
      cv.putText(frame, area, (int(x + radius), int(y + radius)), font, 0.5, (0, 255, 0), 2, cv.LINE_AA)
      cv.circle(frame, center, radius, (0, 255, 0), 2)
      



from enum import Enum
import cv2 as cv
import time

from .process.detect_objects import detect_objects_in_frame, draw_contours


class DataFields:
  objects = "objects"    
class Video:

    def __init__(self, path: str):
        self.path = path
        self.cap = cv.VideoCapture(self.path, apiPreference=cv.CAP_FFMPEG)
        self.data = {
            DataFields.objects: [],
        }

        if not self.cap.isOpened():
            print(f"No se pudo abrir el vídeo \"{self.path}\"")
            exit(1)

    def numOfFrames(self):
        return int(self.cap.get(cv.CAP_PROP_FRAME_COUNT))

    def __str__(self) -> str:
        return self.path

    def reset(self):
        self.cap.set(cv.CAP_PROP_POS_FRAMES, 0)

    def process(self, showContours: bool = False):
        print("Procesando vídeo...")

        cap = self.cap
        self.data[DataFields.objects] = []  # Reset contours_per_frame
        frameRate = int(self.getFrameRate())

        while cap.isOpened():
            ret, frame = cap.read()
            if not ret:
                break

            # Detect objects
            # st = time.time()


            contours = detect_objects_in_frame(frame)
            objects = map(lambda cnt: {"circle": cv.minEnclosingCircle(cnt), "area": cv.contourArea(cnt)}, contours)

            # et = time.time()
            # print(f"Tiempo de ejecución: {et - st}")
            self.data[DataFields.objects].append(objects)

            # Draw contours
            if showContours:
                drawed = frame.copy()
                draw_contours(drawed, contours)
                cv.imshow("CC", drawed)
                if cv.waitKey(frameRate) & 0xFF == ord('q'):
                    break

        
        if (showContours):
            cv.destroyAllWindows()
        print("Vídeo procesado")

    def getConnectedComponents(self):
        return self.data["connected_components"]

    def getFrameRate(self):
        if self.cap:
          return self.cap.get(cv.CAP_PROP_FPS)
        return None